@@ -21,6 +21,7 @@
 #include <qfile.h>
 #include <qregexp.h>
 #include <qtextstream.h>
+#include <qprocess.h>
 
 // ----------------------------------------------------------------------------
 // KDE Headers
@@ -32,6 +33,8 @@
 #include <kapplication.h>
 #include <kconfig.h>
 #include <kglobal.h>
+#include <kstandarddirs.h>
+#include <kcalendarsystem.h>
 
 // ----------------------------------------------------------------------------
 // Project Headers
@@ -40,9 +43,15 @@
 #include "mymoneyqifprofile.h"
 #include "webpricequote.h"
 
+// define static members
+QString WebPriceQuote::m_financeQuoteScriptPath;
+QStringList WebPriceQuote::m_financeQuoteSources;
+
 WebPriceQuote::WebPriceQuote( QObject* _parent, const char* _name ):
   QObject( _parent, _name )
 {
+  m_financeQuoteScriptPath =
+      KGlobal::dirs()->findResource("appdata", QString("misc/financequote.pl"));
   connect(&m_filter,SIGNAL(processExited(const QString&)),this,SLOT(slotParseQuote(const QString&)));
 }
 
@@ -52,6 +61,13 @@ WebPriceQuote::~WebPriceQuote()
 
 bool WebPriceQuote::launch( const QString& _symbol, const QString& _id, const QString& _sourcename )
 {
+  if (_sourcename.contains("Finance::Quote"))
+    return (launchFinanceQuote (_symbol, _id, _sourcename));
+  else
+    return (launchNative (_symbol, _id, _sourcename));
+}
+
+bool WebPriceQuote::launchNative( const QString& _symbol, const QString& _id, const QString& _sourcename ) {
   bool result = true;
   m_symbol = _symbol;
   m_id = _id;
@@ -166,6 +182,47 @@ bool WebPriceQuote::launch( const QString& _symbol, const QString& _id, const QS
   return result;
 }
 
+bool WebPriceQuote::launchFinanceQuote ( const QString& _symbol, const QString& _id,
+                               const QString& _sourcename ) {
+  bool result = true;
+  m_symbol = _symbol;
+  m_id = _id;
+  QString FQSource = _sourcename.section (" ", 1);
+  m_source = WebPriceQuoteSource (_sourcename, m_financeQuoteScriptPath,
+                                  "\"([^,\"]*)\",.*",  // symbol regexp
+                                  "[^,]*,[^,]*,\"([^\"]*)\"", // price regexp
+                                  "[^,]*,([^,]*),.*", // date regexp
+                                  "%y-%m-%d"); // date format
+
+  //emit status(QString("(Debug) symbol=%1 id=%2...").arg(_symbol,_id));
+
+
+  m_filter.clearArguments();
+  m_filter << "perl" << m_financeQuoteScriptPath << FQSource << KProcess::quote(_symbol);
+  m_filter.setUseShell(true);
+  m_filter.setSymbol(m_symbol);
+  emit status(QString("Executing %1 %2 %3...").arg(m_financeQuoteScriptPath).arg(FQSource).arg(_symbol));
+
+    // if we're running non-interactive, we'll need to block.
+    // otherwise, just let us know when it's done.
+  KProcess::RunMode mode = KProcess::NotifyOnExit;
+  if ( ! kapp )
+    mode = KProcess::Block;
+
+  if(m_filter.start(mode, KProcess::All))
+  {
+    result = true;
+    m_filter.resume();
+  }
+  else
+  {
+    emit error(QString("Unable to launch: %1").arg(m_financeQuoteScriptPath));
+    slotParseQuote(QString());
+  }
+
+  return result;
+}
+
 void WebPriceQuote::slotParseQuote(const QString& _quotedata)
 {
   QString quotedata = _quotedata;
@@ -248,7 +305,9 @@ void WebPriceQuote::slotParseQuote(const QString& _quotedata)
       }
       catch (MyMoneyException* e)
       {
-        emit error(i18n("Unable to parse date %1 using format %2: %3").arg(datestr,dateparse.format(),e->what()));
+        // emit error(i18n("Unable to parse date %1 using format %2: %3").arg(datestr,dateparse.format(),e->what()));
+        m_date = QDate::currentDate();
+        gotdate = true;
         delete e;
       }
     }
@@ -297,13 +356,14 @@ QMap<QString,WebPriceQuoteSource> WebPriceQuote::defaultQuoteSources(void)
     "[^,]*,[^,]*,\"([^\"]*)\"", // dateregexp
     "%m %d %y" // dateformat
   );
-
+  // sl1d1 format for Yahoo France doesn't seem to give a date ever
+  // sl1d3 gives us time (99h99) and date
   result["Yahoo France"] = WebPriceQuoteSource("Yahoo France",
-    "http://fr.finance.yahoo.com/d/quotes.csv?s=%251&f=sl1d1",
+    "http://fr.finance.yahoo.com/d/quotes.csv?s=%1&f=sl1d3",
     "([^;]*).*",             // symbolregexp
     "[^;]*.([^;]*),*",       // priceregexp
-    "[^;]*.[^;]*.([^;]*),*", // dateregexp
-    "%m %d %y"               // dateformat
+    "[^;]*.[^;]*...h...([^;]*)", // dateregexp
+    "%d/%m/%y"               // dateformat
   );
 
   result["Globe & Mail"] = WebPriceQuoteSource("Globe & Mail",
@@ -321,13 +381,21 @@ QMap<QString,WebPriceQuoteSource> WebPriceQuote::defaultQuoteSources(void)
     "NAV update (\\d+\\D+\\d+\\D+\\d+)", // dateregexp
     "%d %m %y" // dateformat
   );
-
-  result["VWD.DE"] = WebPriceQuoteSource("VWD.DE",
-    "http://www.finanztreff.de/ftreff/kurse_einzelkurs_uebersicht.htm?s=%1",
+  // Finanztreff (replaces VWD.DE) and boerseonline supplied by Micahel Zimmerman
+  result["Finanztreff"] = WebPriceQuoteSource("Finanztreff",
+    "http://finanztreff.de/kurse_einzelkurs_detail.htn?u=100&i=%1",
     QString(),  // symbolregexp
-    "[Kurs|Realtime]: \\d+\\D+\\d+\\D+\\d+ ([0-9.]+,\\d+)", // priceregexp
-    "Jahreschart (\\d+\\D+\\d+\\D+\\d+)", // dateregexp
-    "%d %m %y" // dateformat
+    "([0-9]+,\\d+).+Gattung:Fonds", // priceregexp
+    "\\).(\\d+\\D+\\d+\\D+\\d+)", // dateregexp (doesn't work; date in chart
+    "%d.%m.%y" // dateformat
+  );
+
+  result["boerseonline"] = WebPriceQuoteSource("boerseonline",
+    "http://www.boerse-online.de/tools/boerse/einzelkurs_kurse.htm?&s=%1",
+    QString(),  // symbolregexp
+    "Akt\\. Kurs.(\\d+,\\d\\d)", // priceregexp
+    "Datum.(\\d+\\.\\d+\\.\\d+)", // dateregexp (doesn't work; date in chart
+    "%d.%m.%y" // dateformat
   );
 
   // The following two price sources were contributed by
@@ -341,6 +409,27 @@ QMap<QString,WebPriceQuoteSource> WebPriceQuote::defaultQuoteSources(void)
     "%d %m %y" // dateformat
   );
 
+  // This quote source provided by Peter Lord
+  // The trading symbol will normally be the SEDOL (see wikipedia) but
+  // the flexibility presently (1/2008) in the code will allow use of
+  // the ISIN or MEXID (FT specific) codes
+  result["Financial Times UK Funds"] = WebPriceQuoteSource("Financial Times UK Funds",
+      "http://funds.ft.com/funds/simpleSearch.do?searchArea=%&search=%1",
+      "SEDOL[\\ ]*(\\d+.\\d+)", // symbol regexp
+      "\\(GBX\\)[\\ ]*([0-9,]*.\\d+)[\\ ]*", // price regexp
+      "Valuation date:[\\ ]*(\\d+/\\d+/\\d+)", // date regexp
+      "%d/%m/%y" // date format
+  );
+
+  // This quote source provided by Danny Scott
+  result["Yahoo Canada"] = WebPriceQuoteSource("Yahoo Canada",
+      "http://ca.finance.yahoo.com/q?s=%1",
+      "%1", // symbol regexp
+      "Last Trade: (\\d+\\.\\d+)", // price regexp
+      "day, (.\\D+\\d+\\D+\\d+)", // date regexp
+      "%m %d %y" // date format
+  );
+
   // (tf2k) The "mpid" is I think the market place id. In this case five
   // stands for Hamburg.
   //
@@ -349,9 +438,9 @@ QMap<QString,WebPriceQuoteSource> WebPriceQuote::defaultQuoteSources(void)
   // Xetra, 32 NASDAQ, 36 NYSE
 
   result["Wallstreet-Online.DE (Hamburg)"] = WebPriceQuoteSource("Wallstreet-Online.DE (Hamburg)",
-    "http://www.wallstreet-online.de/si/?k=%1&spid=ws&mpid=5",
+    "http://fonds.wallstreet-online.de/si/?k=%1&spid=ws&mpid=5",
     "Symbol:(\\w+)",  // symbolregexp
-    "Letzter Kurs: ([0-9.]+,\\d+)", // priceregexp
+    "Fonds \\(EUR\\) ([0-9.]+,\\d+)", // priceregexp
     ", (\\d+\\D+\\d+\\D+\\d+)", // dateregexp
     "%d %m %y" // dateformat
   );
@@ -369,16 +458,59 @@ QMap<QString,WebPriceQuoteSource> WebPriceQuote::defaultQuoteSources(void)
 
   result["Gielda Papierow Wartosciowych (GPW)"] = WebPriceQuoteSource("Gielda Papierow Wartosciowych (GPW)",
     "http://stooq.com/q/?s=%1",
-    QString(),                      // symbol regexp
+    QString(),                   // symbol regexp
     "Kurs.*(\\d+\\.\\d+).*Data",    // price regexp
     "(\\d{4,4}-\\d{2,2}-\\d{2,2})", // date regexp
-    "%y %m %d"                      // date format
+    "%y %m %d"                   // date format
+  );
+
+  // The following price quote is for getting prices of different funds
+  // at OMX Baltic market.
+  result["OMX Baltic funds"] = WebPriceQuoteSource("OMX Baltic funds",
+      "http://www.baltic.omxgroup.com/market/?pg=nontradeddetails&currency=0&instrument=%1",
+      QString(),  // symbolregexp
+      "NAV (\\d+,\\d+)",  // priceregexp
+      "Kpv (\\d+.\\d+.\\d+)",  // dateregexp
+      "%d.%m.%y"   // dateformat
+  );
+
+  // The following price quote was contributed by
+  // Peter Hargreaves <pete.h@pdh-online.info>
+  // The original posting can be found here:
+  // http://sourceforge.net/mailarchive/message.php?msg_name=200806060854.11682.pete.h%40pdh-online.info
+
+  // I have PEP and ISA accounts which I invest in Funds with Barclays 
+  // Stockbrokers. They give me Fund data via Financial Express:
+  // 
+  // https://webfund6.financialexpress.net/Clients/Barclays/default.aspx
+  //
+  // A typical Fund Factsheet is:
+  //
+  // https://webfund6.financialexpress.net/Clients/Barclays/search_factsheet_summary.aspx?code=0585239
+  //
+  // On the Factsheet to identify the fund you can see ISIN Code GB0005852396. 
+  // In the url, this code is shortened by loosing the first four and last 
+  // characters.
+
+  result["Financial Express"] = WebPriceQuoteSource("Financial Express",
+       "https://webfund6.financialexpress.net/Clients/Barclays/search_factsheet_summary.aspx?code=%1",
+       "ISIN Code[^G]*(GB..........).*",  // symbolregexp
+       "[Bid|Mid|Offer] Price[^0-9]*([0-9,\\.]+)p.*", // priceregexp
+       "Price Date[^0-9]*(../../....).*", // dateregexp
+       "%d/%m/%y"                         // dateformat
   );
 
   return result;
 }
 
-QStringList WebPriceQuote::quoteSources(void)
+QStringList WebPriceQuote::quoteSources (const _quoteSystemE _system) {
+  if (_system == Native)
+    return (quoteSourcesNative());
+  else
+    return (quoteSourcesFinanceQuote());
+}
+
+QStringList WebPriceQuote::quoteSourcesNative()
 {
   KConfig *kconfig = KGlobal::config();
   QStringList groups = kconfig->groupList();
@@ -433,6 +565,21 @@ QStringList WebPriceQuote::quoteSources(void)
   return groups;
 }
 
+QStringList WebPriceQuote::quoteSourcesFinanceQuote()
+{
+  if (m_financeQuoteSources.empty()) { // run the process one time only
+    FinanceQuoteProcess getList;
+    m_financeQuoteScriptPath =
+        KGlobal::dirs()->findResource("appdata", QString("misc/financequote.pl"));
+    getList.launch( m_financeQuoteScriptPath );
+    while (!getList.isFinished()) {
+      qApp->processEvents();
+    }
+    m_financeQuoteSources = getList.getSourceList();
+  }
+  return (m_financeQuoteSources);
+}
+
 //
 // Helper class to load/save an individual source
 //
@@ -499,18 +646,135 @@ void WebPriceQuoteProcess::slotReceivedDataFromFilter(KProcess* /*_process*/, ch
   data.duplicate(_pcbuffer, _nbufferlen);
 
 //   kdDebug(2) << "WebPriceQuoteProcess::slotReceivedDataFromFilter(): " << QString(data) << endl;
-
   m_string += QString(data);
 }
 
 void WebPriceQuoteProcess::slotProcessExited(KProcess*)
 {
 //   kdDebug(2) << "WebPriceQuoteProcess::slotProcessExited()" << endl;
-
   emit processExited(m_string);
   m_string.truncate(0);
 }
 
+//
+// Helper class to babysit the KProcess used for running the Finance Quote sources script
+//
+
+FinanceQuoteProcess::FinanceQuoteProcess(void)
+{
+  m_isDone = false;
+  m_string = "";
+  m_fqNames["aex"] = "AEX";
+  m_fqNames["aex_futures"] = "AEX Futures";
+  m_fqNames["aex_options"] = "AEX Options";
+  m_fqNames["amfiindia"] = "AMFI India";
+  m_fqNames["asegr"] = "ASE";
+  m_fqNames["asia"] = "Asia (Yahoo, ...)";
+  m_fqNames["asx"] = "ASX";
+  m_fqNames["australia"] = "Australia (ASX, Yahoo, ...)";
+  m_fqNames["bmonesbittburns"] = "BMO NesbittBurns";
+  m_fqNames["brasil"] = "Brasil (Yahoo, ...)";
+  m_fqNames["canada"] = "Canada (Yahoo, ...)";
+  m_fqNames["canadamutual"] = "Canada Mutual (Fund Library, ...)";
+  m_fqNames["deka"] = "Deka Investments";
+  m_fqNames["dutch"] = "Dutch (AEX, ...)";
+  m_fqNames["dwsfunds"] = "DWS";
+  m_fqNames["europe"] = "Europe (Yahoo, ...)";
+  m_fqNames["fidelity"] = "Fidelity (Fidelity, ...)";
+  m_fqNames["fidelity_direct"] = "Fidelity Direct";
+  m_fqNames["financecanada"] = "Finance Canada";
+  m_fqNames["ftportfolios"] = "First Trust (First Trust, ...)";
+  m_fqNames["ftportfolios_direct"] = "First Trust Portfolios";
+  m_fqNames["fundlibrary"] = "Fund Library";
+  m_fqNames["greece"] = "Greece (ASE, ...)";
+  m_fqNames["indiamutual"] = "India Mutual (AMFI, ...)";
+  m_fqNames["maninv"] = "Man Investments";
+  m_fqNames["fool"] = "Motley Fool";
+  m_fqNames["nasdaq"] = "Nasdaq (Yahoo, ...)";
+  m_fqNames["nz"] = "New Zealand (Yahoo, ...)";
+  m_fqNames["nyse"] = "NYSE (Yahoo, ...)";
+  m_fqNames["nzx"] = "NZX";
+  m_fqNames["platinum"] = "Platinum Asset Management";
+  m_fqNames["seb_funds"] = "SEB";
+  m_fqNames["sharenet"] = "Sharenet";
+  m_fqNames["za"] = "South Africa (Sharenet, ...)";
+  m_fqNames["troweprice_direct"] = "T. Rowe Price";
+  m_fqNames["troweprice"] = "T. Rowe Price";
+  m_fqNames["tdefunds"] = "TD Efunds";
+  m_fqNames["tdwaterhouse"] = "TD Waterhouse Canada";
+  m_fqNames["tiaacref"] = "TIAA-CREF";
+  m_fqNames["trustnet"] = "Trustnet";
+  m_fqNames["uk_unit_trusts"] = "U.K. Unit Trusts";
+  m_fqNames["unionfunds"] = "Union Investments";
+  m_fqNames["tsp"] = "US Govt. Thrift Savings Plan";
+  m_fqNames["usfedbonds"] = "US Treasury Bonds";
+  m_fqNames["usa"] = "USA (Yahoo, Fool ...)";
+  m_fqNames["vanguard"] = "Vanguard";
+  m_fqNames["vwd"] = "VWD";
+  m_fqNames["yahoo"] = "Yahoo";
+  m_fqNames["yahoo_asia"] = "Yahoo Asia";
+  m_fqNames["yahoo_australia"] = "Yahoo Australia";
+  m_fqNames["yahoo_brasil"] = "Yahoo Brasil";
+  m_fqNames["yahoo_europe"] = "Yahoo Europe";
+  m_fqNames["yahoo_nz"] = "Yahoo New Zealand";
+  m_fqNames["zifunds"] = "Zuerich Investments";
+  connect(this, SIGNAL(receivedStdout(KProcess*, char*, int)), this, SLOT(slotReceivedDataFromFilter(KProcess*, char*, int)));
+  connect(this, SIGNAL(processExited(KProcess*)), this, SLOT(slotProcessExited(KProcess*)));
+}
+
+void FinanceQuoteProcess::slotReceivedDataFromFilter(KProcess* /*_process*/, char* _pcbuffer, int _nbufferlen)
+{
+  QByteArray data;
+  data.duplicate(_pcbuffer, _nbufferlen);
+
+//   kdDebug(2) << "WebPriceQuoteProcess::slotReceivedDataFromFilter(): " << QString(data) << endl;
+  m_string += QString(data);
+}
+
+void FinanceQuoteProcess::slotProcessExited(KProcess*)
+{
+//   kdDebug(2) << "WebPriceQuoteProcess::slotProcessExited()" << endl;
+  m_isDone = true;
+}
+
+void FinanceQuoteProcess::launch (const QString& scriptPath) {
+  clearArguments();
+  arguments.append(QCString("perl"));
+  arguments.append (QCString(scriptPath));
+  arguments.append (QCString("-l"));
+  if (!start(KProcess::NotifyOnExit, KProcess::Stdout)) qFatal ("Unable to start FQ script");
+  return;
+}
+
+QStringList FinanceQuoteProcess::getSourceList() {
+  QStringList raw = QStringList::split(0x0A, m_string);
+  QStringList sources;
+  QStringList::iterator it;
+  for (it = raw.begin(); it != raw.end(); ++it) {
+    if (m_fqNames[*it].isEmpty()) sources.append(*it);
+    else sources.append(m_fqNames[*it]);
+  }
+  sources.sort();
+  return (sources);
+}
+
+const QString FinanceQuoteProcess::crypticName(const QString& niceName) {
+  QString ret (niceName);
+  fqNameMap::iterator it;
+  for (it = m_fqNames.begin(); it != m_fqNames.end(); ++it) {
+    if (niceName == it.data()) {
+      ret = it.key();
+      break;
+    }
+  }
+  return (ret);
+}
+
+const QString FinanceQuoteProcess::niceName(const QString& crypticName) {
+  QString ret (m_fqNames[crypticName]);
+  if (ret.isEmpty()) ret = crypticName;
+  return (ret);
+}
 //
 // Universal date converter
 //
@@ -595,7 +859,8 @@ QDate MyMoneyDateFormat::convertString(const QString& _in, bool _strict, unsigne
         unsigned i = 1;
         while ( i <= 12 )
         {
-          if (  QDate::shortMonthName(i).lower() == *it_scanned || QDate::longMonthName(i).lower() == *it_scanned )
+          if(KGlobal::locale()->calendar()->monthName(i, 2000, true).lower() == *it_scanned
+          || KGlobal::locale()->calendar()->monthName(i, 2000, false).lower() == *it_scanned)
             month = i;
           ++i;
         }
