@@ -21,10 +21,15 @@
  *                                                                         *
  ***************************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "kdecompat.h"
 
 // ----------------------------------------------------------------------------
 // QT Includes
+
 #include <qlayout.h>
 #include <qdatetime.h>
 #include <qregexp.h>
@@ -37,10 +42,10 @@
 #include <qtimer.h>
 #include <qiconset.h>
 #include <qpopupmenu.h>
-#include <qmessagebox.h>
 #include <qpushbutton.h>
 #include <qtooltip.h>
-
+#include <qcheckbox.h>
+#include <qvbox.h>
 // ----------------------------------------------------------------------------
 // KDE Includes
 
@@ -58,13 +63,15 @@
 
 // ----------------------------------------------------------------------------
 // Project Includes
+
+#include <kmymoney/mymoneyfile.h>
+#include <kmymoney/mymoneyreport.h>
+#include <kmymoney/kmymoneyglobalsettings.h>
+
 #include "kreportsview.h"
-#include "../reports/pivottable.h"
 #include "../reports/querytable.h"
-#include "../mymoney/mymoneyfile.h"
-#include "../mymoney/mymoneyreport.h"
+#include "../reports/objectinfotable.h"
 #include "../dialogs/kreportconfigurationfilterdlg.h"
-#include "../widgets/kmymoneytitlelabel.h"
 #include "../kmymoneyutils.h"
 
 using namespace reports;
@@ -87,13 +94,18 @@ KReportsView::KReportTab::KReportTab(KTabWidget* parent, const MyMoneyReport& re
   m_layout( new QVBoxLayout( this, 11, 6, "reporttablayout" ) ),
   m_report( report ),
   m_deleteMe( false ),
-  m_showingChart( false )
+  m_showingChart( false ),
+  m_needReload( true ),
+  m_table(0)
 {
-  if ( ! KReportChartView::implemented() )
+  m_part->setZoomFactor( KMyMoneyGlobalSettings::fontSizePercentage() );
+
+  if ( ! KReportChartView::implemented() || m_report.reportType() != MyMoneyReport::ePivotTable )
   {
     m_control->buttonChart->hide();
   }
 
+  m_chartView->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
   m_chartView->hide();
   m_layout->addWidget( m_control ); //, 0, Qt::AlignTop );
   m_layout->addWidget( m_part->view() );
@@ -107,6 +119,16 @@ KReportsView::KReportTab::KReportTab(KTabWidget* parent, const MyMoneyReport& re
 
   parent->insertTab( this, QIconSet(QPixmap(icon)), report.name() );
   parent->setTabEnabled( this, true );
+
+#ifdef HAVE_KDCHART
+  if ( m_report.isChartByDefault() )
+    toggleChart();
+#endif
+}
+
+KReportsView::KReportTab::~KReportTab()
+{
+  delete m_table;
 }
 
 void KReportsView::KReportTab::print(void)
@@ -122,34 +144,98 @@ void KReportsView::KReportTab::copyToClipboard(void)
   QApplication::clipboard()->setData(pdrag);
 }
 
-void KReportsView::KReportTab::saveAs( const QString& filename )
+void KReportsView::KReportTab::saveAs( const QString& filename, bool includeCSS )
 {
   QFile file( filename );
   if ( file.open( IO_WriteOnly ) )
   {
     if ( QFileInfo(filename).extension(false).lower() == "csv")
     {
-      if ( m_report.reportType() == MyMoneyReport::ePivotTable )
-        QTextStream(&file) << PivotTable( m_report ).renderCSV();
-      else if ( m_report.reportType() == MyMoneyReport::eQueryTable )
-        QTextStream(&file) << QueryTable( m_report ).renderCSV();
+      QTextStream(&file) << m_table->renderCSV();
+    }
+    else {
+      QTextStream stream(&file);
+      QRegExp exp("(.*)(<link.*css\" href=)\"(.*)\">(<meta.*utf-8\" />)(.*)");
+      QString table = createTable();
+      if(exp.search(table) != -1 && includeCSS) {
+        QFile cssFile(exp.cap(3));
+        if(cssFile.open(IO_ReadOnly)) {
+          QTextStream cssStream(&cssFile);
+          stream << exp.cap(1);
+          stream << exp.cap(4);
+          stream << endl << "<style type=\"text/css\">" << endl << "<!--" << endl;
+          stream << cssStream.read();
+          stream << "-->" << endl << "</style>" << endl;
+          stream << exp.cap(5);
+          cssFile.close();
+        } else {
+          stream << table;
+        }
+      } else {
+        stream << table;
+      }
     }
-    else
-      QTextStream(&file) <<  createTable();
     file.close();
   }
 }
 
+void KReportsView::KReportTab::loadTab(void)
+{
+  m_needReload = true;
+  if(isVisible()) {
+    m_needReload = false;
+    updateReport();
+  }
+}
+
+void KReportsView::KReportTab::show(void)
+{
+  if(m_needReload) {
+    m_needReload = false;
+    updateReport();
+  }
+  QWidget::show();
+}
+
 void KReportsView::KReportTab::updateReport(void)
 {
+  // reload the report from the engine. It might have
+  // been changed by the user
+
+  try {
+    // Don't try to reload default reports from the engine
+    if(!m_report.id().isEmpty())
+      m_report = MyMoneyFile::instance()->report(m_report.id());
+  } catch(MyMoneyException* e) {
+    delete e;
+  }
+
+  delete m_table;
+  m_table = 0;
+
+  if ( m_report.reportType() == MyMoneyReport::ePivotTable ) {
+    m_table = new PivotTable(m_report);
+  } else if ( m_report.reportType() == MyMoneyReport::eQueryTable ) {
+    m_table = new QueryTable(m_report);
+  } else if ( m_report.reportType() == MyMoneyReport::eInfoTable ) {
+    m_table = new ObjectInfoTable(m_report);
+  }
+
   m_part->begin();
   m_part->write(createTable());
   m_part->end();
+
+  m_table->drawChart( *m_chartView );
+  m_chartView->update();
 }
 
 QString KReportsView::KReportTab::createTable(const QString& links)
 {
-  QString filename = KGlobal::dirs()->findResource("appdata", "html/kmymoney2.css");
+  QString filename;
+  if(!MyMoneyFile::instance()->value("reportstylesheet").isEmpty())
+    filename = KGlobal::dirs()->findResource("appdata", QString("html/%1").arg(MyMoneyFile::instance()->value("reportstylesheet")));
+  if(filename.isEmpty())
+    filename = KGlobal::dirs()->findResource("appdata", "html/kmymoney2.css");
   QString header = QString("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\">\n") +
     QString("<html><head><link rel=\"stylesheet\" type=\"text/css\" href=\"%1\">").arg(filename);
 
@@ -165,10 +251,7 @@ QString KReportsView::KReportTab::createTable(const QString& links)
     html += header;
     html += links;
 
-    if ( m_report.reportType() == MyMoneyReport::ePivotTable )
-      html += PivotTable( m_report ).renderHTML();
-    else if ( m_report.reportType() == MyMoneyReport::eQueryTable )
-      html += QueryTable( m_report ).renderHTML();
+    html += m_table->renderHTML();
 
     html += footer;
   }
@@ -178,7 +261,7 @@ QString KReportsView::KReportTab::createTable(const QString& links)
 
     QString error = QString(i18n("There was an error creating your report: \"%1\".\nPlease report this error to the developer's list: kmymoney2-developer@lists.sourceforge.net")).arg(e->what());
 
-    QMessageBox::critical(this,i18n("Critical Error"), error, QMessageBox::Ok, QMessageBox::NoButton );
+    KMessageBox::error(this, error, i18n("Critical Error"));
 
     html += header;
     html += links;
@@ -205,10 +288,6 @@ void KReportsView::KReportTab::toggleChart(void)
   }
   else
   {
-    //FIXME: Check the type of report and call the correct report type, don't just assume
-    //it's a pivot table...duh!
-    PivotTable t( m_report );
-    t.drawChart( *m_chartView );
     m_part->hide();
     m_chartView->show();
 
@@ -222,14 +301,23 @@ void KReportsView::KReportTab::toggleChart(void)
   * KReportsView Implementation
   */
 
-KReportsView::KReportsView(QWidget *parent, const char *name )
- : KMyMoneyViewBase(parent, name, i18n("Reports"))
+class KReportsViewPrivate
+{
+public:
+  KReportsViewPrivate() :
+    includeCSS(0) {}
+
+  QCheckBox* includeCSS;
+};
+
+KReportsView::KReportsView(QWidget *parent, const char *name ) :
+  KMyMoneyViewBase(parent, name, i18n("Reports")),
+  d(new KReportsViewPrivate),
+  m_needReload(false)
 {
   m_reportTabWidget = new KTabWidget( this, "m_reportTabWidget" );
   m_viewLayout->addWidget( m_reportTabWidget );
-#if KDE_IS_VERSION(3,2,0)
   m_reportTabWidget->setHoverCloseButton( true );
-#endif
 
   m_listTab = (new QWidget( m_reportTabWidget, "indextab" ));
   m_listTabLayout = ( new QVBoxLayout( m_listTab, 11, 6, "indextabLayout") );
@@ -242,35 +330,97 @@ KReportsView::KReportsView(QWidget *parent, const char *name )
   m_reportListView->setResizeMode(QListView::AllColumns);
   m_reportListView->setAllColumnsShowFocus(true);
   m_reportListView->setRootIsDecorated(true);
+  m_reportListView->setShadeSortColumn(false);
 
-#if KDE_IS_VERSION(3,2,0)
   connect( m_reportTabWidget, SIGNAL(closeRequest(QWidget*)),
     this, SLOT(slotClose(QWidget*)) );
-#endif
   connect(m_reportListView, SIGNAL(doubleClicked(QListViewItem*)),
     this, SLOT(slotOpenReport(QListViewItem*)));
   connect(m_reportListView, SIGNAL(returnPressed(QListViewItem*)),
     this, SLOT(slotOpenReport(QListViewItem*)));
   connect( m_reportListView, SIGNAL(contextMenu(KListView*,QListViewItem*,const QPoint &)),
     this, SLOT(slotListContextMenu(KListView*,QListViewItem*,const QPoint &)));
+
+  connect(MyMoneyFile::instance(), SIGNAL(dataChanged()), this, SLOT(slotLoadView()));
 }
 
 KReportsView::~KReportsView()
 {
+  delete d;
 }
 
 void KReportsView::show()
 {
-  slotRefreshView();
-  QWidget::show();
-  emit signalViewActivated();
+  if(m_needReload) {
+    loadView();
+    m_needReload = false;
+  }
+
+  KReportTab* tab = dynamic_cast<KReportTab*>(m_reportTabWidget->currentPage());
+  if(tab)
+    emit reportSelected(tab->report());
+  else
+    emit reportSelected(MyMoneyReport());
+
+  // don't forget base class implementation
+  KMyMoneyViewBase::show();
 }
 
-void KReportsView::slotRefreshView(void)
+void KReportsView::slotLoadView(void)
 {
-  // set fonts according configuration
-  m_reportListView->header()->setFont(KMyMoneyUtils::headerFont());
-  m_reportListView->setFont(KMyMoneyUtils::cellFont());
+  m_needReload = true;
+  if(isVisible()) {
+    loadView();
+    m_needReload = false;
+  }
+}
+
+QString KReportsView::KReportGroupListItem::key ( int column, bool ascending ) const
+{
+  if (column == 0)
+    return QString::number(m_nr).rightJustify(3,'0');
+  else
+    return KListViewItem::key(column,ascending);
+}
+
+KReportsView::KReportGroupListItem::KReportGroupListItem(KListView* parent, const int nr, QString name) :
+  KListViewItem(parent),
+  m_name(name)
+{
+  setNr(nr);
+}
+
+void KReportsView::KReportGroupListItem::setNr(const int nr)
+{
+  m_nr = nr;
+  setText(0, QString("%1. %2").arg(nr).arg(m_name));
+}
+
+void KReportsView::loadView(void)
+{
+  ::timetrace("Start KReportsView::loadView");
+
+  // remember the id of the current selected item and the
+  // items that are shown 'expanded'
+  QMap<QString, bool> isOpen;
+  QListViewItem *item = m_reportListView->selectedItem();
+  QString selectedPage = (item) ? item->text(0) : QString();
+
+  // keep a map of all 'expanded' accounts
+  QListViewItemIterator it_lvi(m_reportListView);
+  while(it_lvi.current()) {
+    item = it_lvi.current();
+    if(item && item->isOpen()) {
+      isOpen[item->text(0)] = true;
+    }
+    ++it_lvi;
+  }
+
+  // remember the upper left corner of the viewport
+  QPoint startPoint = m_reportListView->viewportToContents(QPoint(0, 0));
+
+  // turn off updates to avoid flickering during reload
+  m_reportListView->setUpdatesEnabled(false);
 
   //
   // Rebuild the list page
@@ -279,16 +429,17 @@ void KReportsView::slotRefreshView(void)
   unsigned pagenumber = 1;
 
   // Default Reports
+  KReportGroupListItem* chartnode = new KReportGroupListItem(m_reportListView, 10, i18n("Charts"));
 
-  QMap<QString,KListViewItem*> groupitems;
+  QMap<QString,KReportGroupListItem*> groupitems;
   QValueList<ReportGroup> defaultreports;
   defaultReports(defaultreports);
   QValueList<ReportGroup>::const_iterator it_group = defaultreports.begin();
   while ( it_group != defaultreports.end() )
   {
     QString groupname = (*it_group).name();
-    QString pagename = QString::number(pagenumber++) + ". " + groupname;
-    KListViewItem* curnode = new KListViewItem(m_reportListView,pagename);
+    KReportGroupListItem* curnode = new KReportGroupListItem(m_reportListView, pagenumber++, (*it_group).title());
+    curnode->setOpen(isOpen.find(curnode->text(0)) != isOpen.end());
     groupitems[groupname] = curnode;
 
     QValueList<MyMoneyReport>::const_iterator it_report = (*it_group).begin();
@@ -296,34 +447,43 @@ void KReportsView::slotRefreshView(void)
     {
       MyMoneyReport report = *it_report;
       report.setGroup(groupname);
-      new KReportListItem( curnode, report );
+      KReportListItem* r = new KReportListItem( curnode, report );
+      if(report.name() == selectedPage)
+        m_reportListView->setSelected(r, true);
+
+      // ALSO place it into the Charts list if it's displayed as a chart by default
+      if ( (*it_report).isChartByDefault() )
+        new KReportListItem( chartnode, *it_report );
+
       ++it_report;
     }
 
     ++it_group;
   }
+
+  // Rename the charts item to place it at this point in the list.
+  chartnode->setNr(pagenumber++);
+  chartnode->setOpen(isOpen.find(chartnode->text(0)) != isOpen.end());
+
   // Custom reports
 
-  QString pagename = QString::number(pagenumber++) + ". " + i18n("Favorite Reports");
-  KListViewItem* favoritenode = new KListViewItem(m_reportListView,pagename);
-  KListViewItem* orphannode = NULL;
+  KReportGroupListItem* favoritenode = new KReportGroupListItem(m_reportListView,pagenumber++, i18n("Favorite Reports"));
+  favoritenode->setOpen(isOpen.find(favoritenode->text(0)) != isOpen.end());
+  KReportGroupListItem* orphannode = NULL;
 
   QValueList<MyMoneyReport> customreports = MyMoneyFile::instance()->reportList();
   QValueList<MyMoneyReport>::const_iterator it_report = customreports.begin();
   while( it_report != customreports.end() )
   {
     // If this report is in a known group, place it there
-    KListViewItem* groupnode = groupitems[(*it_report).group()];
+    KReportGroupListItem* groupnode = groupitems[(*it_report).group()];
     if ( groupnode )
       new KReportListItem( groupnode, *it_report );
     else
     // otherwise, place it in the orphanage
     {
       if ( ! orphannode )
-      {
-        QString pagename = QString::number(pagenumber++) + ". " + i18n("Old Customized Reports");
-        orphannode = new KListViewItem(m_reportListView,pagename);
-      }
+        orphannode = new KReportGroupListItem(m_reportListView, pagenumber++, i18n("Old Customized Reports"));
       new KReportListItem( orphannode, *it_report );
     }
 
@@ -331,11 +491,22 @@ void KReportsView::slotRefreshView(void)
     if ( (*it_report).isFavorite() )
       new KReportListItem( favoritenode, *it_report );
 
+    // ALSO place it into the Charts list if it's displayed as a chart by default
+    if ( (*it_report).isChartByDefault() )
+      new KReportListItem( chartnode, *it_report );
+
     ++it_report;
   }
 
+  // reposition viewport
+  m_reportListView->setContentsPos(startPoint.x(), startPoint.y());
+
+  // turn updates back on
+  m_reportListView->setUpdatesEnabled(true);
+  m_reportListView->repaintContents();
+
   //
-  // Go through the tabs to update all the reports, or delete them if needed
+  // Go through the tabs to set their update flag or delete them if needed
   //
 
   int index = 1;
@@ -349,22 +520,22 @@ void KReportsView::slotRefreshView(void)
       --index;
     }
     else
-      tab->updateReport();
+      tab->loadTab();
     ++index;
   }
+  ::timetrace("Done KReportsView::loadView");
 }
 
 void KReportsView::slotOpenURL(const KURL &url, const KParts::URLArgs& /* args */)
 {
   QString view = url.fileName(false);
-  QCString id = url.queryItem("id").data();
   QCString command = url.queryItem("command").data();
 
   if(view == VIEW_REPORTS) {
 
-    if ( command.isEmpty() )
-      slotRefreshView();
-    else if ( command == "print" )
+    if ( command.isEmpty() ) {
+      // slotRefreshView();
+    } else if ( command == "print" )
       slotPrintView();
     else if ( command == "copy" )
       slotCopyView();
@@ -404,25 +575,48 @@ void KReportsView::slotSaveView(void)
 {
   KReportTab* tab = dynamic_cast<KReportTab*>(m_reportTabWidget->currentPage());
   if(tab) {
-    QString newName=KFileDialog::getSaveFileName(KGlobalSettings::documentPath(),i18n("*.csv|CSV files\n""*.html|HTML files\n""*.*|All files"), this, i18n("Save as..."));
+    QVBox* vbox = new QVBox();
+    d->includeCSS = new QCheckBox(i18n("Include Stylesheet"), vbox);
 
-    if(!newName.isEmpty())
-    {
-      if(newName.findRev('.') == -1)
-        newName.append(".html");
+    // the following code is copied from KFileDialog::getSaveFileName,
+    // adjust to our local needs (filetypes etc.) and
+    // enhanced to show the m_saveEncrypted combo box
+    KFileDialog dlg( ":kmymoney-export",
+                   QString("%1|%2\n").arg("*.csv").arg(i18n("CSV (Filefilter)", "CSV files")) +
+                   QString("%1|%2\n").arg("*.html").arg(i18n("HTML (Filefilter)", "HTML files")),
+                   this, "filedialog", true, vbox);
+    connect(&dlg, SIGNAL(filterChanged(const QString&)), this, SLOT(slotSaveFilterChanged(const QString&)));
 
-      tab->saveAs( newName );
+    dlg.setOperationMode( KFileDialog::Saving );
+    dlg.setCaption(i18n("Export as"));
+    slotSaveFilterChanged("*.csv");    // init gui
+
+    if(dlg.exec() == QDialog::Accepted) {
+      KURL newURL = dlg.selectedURL();
+      if (!newURL.isEmpty()) {
+        QString newName = newURL.prettyURL(0, KURL::StripFileProtocol);
+
+        if(newName.findRev('.') == -1)
+          newName.append(".html");
+
+        tab->saveAs( newName, d->includeCSS->isEnabled() && d->includeCSS->isChecked() );
+      }
     }
   }
 }
 
+void KReportsView::slotSaveFilterChanged(const QString& filter)
+{
+  d->includeCSS->setEnabled(filter ==  "*.html");
+}
+
 void KReportsView::slotConfigure(void)
 {
   KReportTab* tab = dynamic_cast<KReportTab*>(m_reportTabWidget->currentPage());
 
   if(tab) {
     MyMoneyReport report = tab->report();
-    if ( report.comment() == i18n("Default Report") )
+    if ( report.comment() == i18n("Default Report") || report.comment() == i18n("Generated Report") )
     {
       report.setComment( i18n("Custom Report") );
       report.setName( report.name() + i18n(" (Customized)") );
@@ -435,11 +629,12 @@ void KReportsView::slotConfigure(void)
       MyMoneyReport newreport = dlg.getConfig();
 
       // If this report has an ID, then MODIFY it, otherwise ADD it
+      MyMoneyFileTransaction ft;
       if ( ! newreport.id().isEmpty() )
       {
         MyMoneyFile::instance()->modifyReport(newreport);
+        ft.commit();
         tab->modifyReport(newreport);
-        slotRefreshView();
 
         m_reportTabWidget->changeTab( tab, newreport.name() );
         m_reportTabWidget->showPage(tab);
@@ -447,6 +642,7 @@ void KReportsView::slotConfigure(void)
       else
       {
         MyMoneyFile::instance()->addReport(newreport);
+        ft.commit();
         new KReportListItem( m_reportListView, newreport );
         addReportTab(newreport);
       }
@@ -463,15 +659,22 @@ void KReportsView::slotDuplicate(void)
     dupe.setName( QString(i18n("Copy of %1")).arg(dupe.name()) );
     if ( dupe.comment() == i18n("Default Report") )
       dupe.setComment( i18n("Custom Report") );
-    dupe.setId(QCString());
+    dupe.clearId();
 
     KReportConfigurationFilterDlg dlg(dupe);
     if (dlg.exec())
     {
       dupe = dlg.getConfig();
-      MyMoneyFile::instance()->addReport(dupe);
-      new KReportListItem( m_reportListView, dupe );
-      addReportTab(dupe);
+      MyMoneyFileTransaction ft;
+      try {
+        MyMoneyFile::instance()->addReport(dupe);
+        ft.commit();
+        new KReportListItem( m_reportListView, dupe );
+        addReportTab(dupe);
+      } catch(MyMoneyException* e) {
+        qDebug("Cannot add report");
+        delete e;
+      }
     }
   }
 }
@@ -484,15 +687,19 @@ void KReportsView::slotDelete(void)
     MyMoneyReport report = tab->report();
     if ( ! report.id().isEmpty() )
     {
-      if ( KMessageBox::Continue == KMessageBox::warningContinueCancel(this, QString(i18n("Are you sure you want to delete %1?  There is no way to recover it!")).arg(report.name()), i18n("Delete Report?")))
+      if ( KMessageBox::Continue == KMessageBox::warningContinueCancel(this, QString("<qt>")+i18n("Are you sure you want to delete report <b>%1</b>?  There is no way to recover it!").arg(report.name())+QString("</qt>"), i18n("Delete Report?")))
       {
-        MyMoneyFile::instance()->removeReport(report);
+        // close the tab and then remove the report so that it is not
+        // generated again during the following loadView() call
         slotClose(tab);
-        slotRefreshView();
+
+        MyMoneyFileTransaction ft;
+        MyMoneyFile::instance()->removeReport(report);
+        ft.commit();
       }
     }
     else
-      KMessageBox::information(this, QString(i18n("Sorry, %1 is a default report.  You may not delete it.")).arg(report.name()), i18n("Delete Report?"));
+      KMessageBox::information(this, QString("<qt>")+i18n("Sorry, <b>%1</b> is a default report.  You may not delete it.").arg(report.name())+QString("</qt>"), i18n("Delete Report?"));
   }
 }
 
@@ -577,6 +784,33 @@ void KReportsView::slotOpenReport(QListViewItem* item)
   }
 }
 
+void KReportsView::slotOpenReport(const MyMoneyReport& report)
+{
+  kdDebug(2) << __func__ << " " << report.name() << endl;
+    KReportTab* page = NULL;
+
+    // Find the tab which contains the report indicated by this list item
+    int index = 1;
+    while ( index < m_reportTabWidget->count() )
+    {
+      KReportTab* current = dynamic_cast<KReportTab*>(m_reportTabWidget->page(index));
+
+        if ( current->report().name() == report.name() )
+        {
+          page = current;
+          break;
+        }
+
+      ++index;
+    }
+
+    // Show the tab, or create a new one, as needed
+    if ( page )
+      m_reportTabWidget->showPage( page );
+    else
+      addReportTab(report);
+}
+
 void KReportsView::slotToggleChart(void)
 {
   KReportTab* tab = dynamic_cast<KReportTab*>(m_reportTabWidget->currentPage());
@@ -634,7 +868,7 @@ void KReportsView::addReportTab(const MyMoneyReport& report)
   if ( report.id().isEmpty() )
     tab->control()->buttonDelete->setEnabled(false);
 
-  slotRefreshView();
+  // slotRefreshView();
 
   m_reportTabWidget->showPage(tab);
 
@@ -697,7 +931,7 @@ void KReportsView::slotDeleteFromList(void)
 void KReportsView::defaultReports(QValueList<ReportGroup>& groups)
 {
   {
-    ReportGroup list = i18n("Income and Expenses");
+    ReportGroup list("Income and Expenses", i18n("Income and Expenses"));
 
     list.push_back(MyMoneyReport(
       MyMoneyReport::eExpenseIncome,
@@ -715,10 +949,39 @@ void KReportsView::defaultReports(QValueList<ReportGroup>& groups)
       i18n("Income and Expenses This Year"),
       i18n("Default Report")
     ));
+
+#ifdef HAVE_KDCHART
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eExpenseIncome,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::yearToDate,
+      true,
+      i18n("Income and Expenses Graph"),
+      i18n("Default Report")
+    ));
+    list.back().setChartByDefault(true);
+    list.back().setDetailLevel(MyMoneyReport::eDetailTop);
+    list.back().setChartType(MyMoneyReport::eChartLine);
+    list.back().setChartDataLabels(false);
+
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eExpenseIncome,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::yearToDate,
+      false,
+      i18n("Income and Expenses Pie Chart"),
+      i18n("Default Report")
+    ));
+    list.back().setChartByDefault(true);
+    list.back().setDetailLevel(MyMoneyReport::eDetailGroup);
+    list.back().setChartType(MyMoneyReport::eChartPie);
+    list.back().setShowingRowTotals(false);
+#endif
+
     groups.push_back(list);
   }
   {
-    ReportGroup list = i18n("Net Worth");
+    ReportGroup list("Net Worth", i18n("Net Worth"));
 
     list.push_back(MyMoneyReport(
       MyMoneyReport::eAssetLiability,
@@ -744,6 +1007,30 @@ void KReportsView::defaultReports(QValueList<ReportGroup>& groups)
       i18n("Net Worth By Year"),
       i18n("Default Report")
     ));
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eAssetLiability,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::next7Days,
+      false,
+      i18n("7-day Cash Flow Forecast"),
+      i18n("Default Report")
+    ));
+    list.back().setIncludingSchedules( true );
+    list.back().setColumnsAreDays( true );
+
+#ifdef HAVE_KDCHART
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eAssetLiability,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::last12Months,
+      false,
+      i18n("Net Worth Graph"),
+      i18n("Default Report")
+    ));
+    list.back().setChartByDefault(true);
+    list.back().setChartGridLines(false);
+    list.back().setDetailLevel(MyMoneyReport::eDetailTotal);
+    list.back().setChartType(MyMoneyReport::eChartLine);
 
     list.push_back(MyMoneyReport(
       MyMoneyReport::eInstitution,
@@ -753,6 +1040,7 @@ void KReportsView::defaultReports(QValueList<ReportGroup>& groups)
       i18n("Account Balances by Institution"),
       i18n("Default Report")
     ));
+#endif
 
     list.push_back(MyMoneyReport(
       MyMoneyReport::eAccountType,
@@ -766,16 +1054,17 @@ void KReportsView::defaultReports(QValueList<ReportGroup>& groups)
     groups.push_back(list);
   }
   {
-    ReportGroup list = i18n("Transactions");
+    ReportGroup list("Transactions", i18n("Transactions"));
 
     list.push_back(MyMoneyReport(
       MyMoneyReport::eAccount,
-      MyMoneyReport::eQCnumber|MyMoneyReport::eQCpayee|MyMoneyReport::eQCcategory,
+      MyMoneyReport::eQCnumber|MyMoneyReport::eQCpayee|MyMoneyReport::eQCcategory|MyMoneyReport::eQCbalance,
       MyMoneyTransactionFilter::yearToDate,
       false,
       i18n("Transactions by Account"),
       i18n("Default Report")
     ));
+    //list.back().setConvertCurrency(false);
     list.push_back(MyMoneyReport(
       MyMoneyReport::eCategory,
       MyMoneyReport::eQCnumber|MyMoneyReport::eQCpayee|MyMoneyReport::eQCaccount,
@@ -808,11 +1097,27 @@ void KReportsView::defaultReports(QValueList<ReportGroup>& groups)
       i18n("Transactions by Week"),
       i18n("Default Report")
     ));
-
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eAccount,
+      MyMoneyReport::eQCloan,
+      MyMoneyTransactionFilter::allDates,
+      false,
+      i18n("Loan Transactions"),
+      i18n("Default Report")
+    ));
+    list.back().setLoansOnly(true);
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eAccountReconcile,
+      MyMoneyReport::eQCnumber|MyMoneyReport::eQCpayee|MyMoneyReport::eQCcategory|MyMoneyReport::eQCbalance,
+      MyMoneyTransactionFilter::last3Months,
+      false,
+      i18n("Transactions by Reconciliation Status"),
+      i18n("Default Report")
+    ));
     groups.push_back(list);
   }
   {
-    ReportGroup list = i18n("Investments");
+    ReportGroup list("Investments", i18n("Investments"));
 
     list.push_back(MyMoneyReport(
       MyMoneyReport::eTopAccount,
@@ -863,21 +1168,233 @@ void KReportsView::defaultReports(QValueList<ReportGroup>& groups)
       i18n("Default Report")
     ));
     list.back().setInvestmentsOnly(true);
+#ifdef HAVE_KDCHART
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eAssetLiability,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::today,
+      false,
+      i18n("Investment Holdings Pie"),
+      i18n("Default Report")
+    ));
+    list.back().setChartByDefault(true);
+    list.back().setChartGridLines(false);
+    list.back().setDetailLevel(MyMoneyReport::eDetailAll);
+    list.back().setChartType(MyMoneyReport::eChartPie);
+    list.back().setInvestmentsOnly(true);
 
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eAssetLiability,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::last12Months,
+      false,
+      i18n("Investment Worth Graph"),
+      i18n("Default Report")
+    ));
+    list.back().setChartByDefault(true);
+    list.back().setChartGridLines(false);
+    list.back().setDetailLevel(MyMoneyReport::eDetailAll);
+    list.back().setChartType(MyMoneyReport::eChartLine);
+    list.back().setColumnsAreDays( true );
+    list.back().setInvestmentsOnly(true);
+
+#endif
     groups.push_back(list);
   }
   {
-    ReportGroup list = i18n("Taxes");
+    ReportGroup list("Taxes", i18n("Taxes"));
 
     list.push_back(MyMoneyReport(
       MyMoneyReport::eCategory,
       MyMoneyReport::eQCnumber|MyMoneyReport::eQCpayee|MyMoneyReport::eQCaccount,
       MyMoneyTransactionFilter::yearToDate,
       false,
-      i18n("Tax Transactions"),
+      i18n("Tax Transactions by Category"),
       i18n("Default Report")
     ));
     list.back().setTax(true);
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::ePayee,
+      MyMoneyReport::eQCnumber|MyMoneyReport::eQCcategory|MyMoneyReport::eQCaccount,
+      MyMoneyTransactionFilter::yearToDate,
+      false,
+      i18n("Tax Transactions by Payee"),
+      i18n("Default Report")
+    ));
+    list.back().setTax(true);
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eCategory,
+      MyMoneyReport::eQCnumber|MyMoneyReport::eQCpayee|MyMoneyReport::eQCaccount,
+      MyMoneyTransactionFilter::lastFiscalYear,
+      false,
+      i18n("Tax Transactions by Category Last Fiscal Year"),
+      i18n("Default Report")
+    ));
+    list.back().setTax(true);
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::ePayee,
+      MyMoneyReport::eQCnumber|MyMoneyReport::eQCcategory|MyMoneyReport::eQCaccount,
+      MyMoneyTransactionFilter::lastFiscalYear,
+      false,
+      i18n("Tax Transactions by Payee Last Fiscal Year"),
+      i18n("Default Report")
+    ));
+    list.back().setTax(true);
+    groups.push_back(list);
+  }
+  {
+    ReportGroup list("Budgeting", i18n("Budgeting"));
+
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eBudgetActual,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::yearToDate,
+      true,
+      i18n("Budgeted vs. Actual This Year"),
+      i18n("Default Report")
+    ));
+    list.back().setShowingRowTotals(true);
+    list.back().setBudget("Any",true);
+
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eBudgetActual,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::currentMonth,
+      true,
+      i18n("Monthly Budgeted vs. Actual"),
+      i18n("Default Report")
+    ));
+    list.back().setBudget("Any",true);
+
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eBudgetActual,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::currentYear,
+      true,
+      i18n("Yearly Budgeted vs. Actual"),
+      i18n("Default Report")
+    ));
+    list.back().setBudget("Any",true);
+    list.back().setShowingRowTotals(true);
+
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eBudget,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::currentMonth,
+      true,
+      i18n("Monthly Budget"),
+      i18n("Default Report")
+    ));
+    list.back().setBudget("Any",false);
+
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eBudget,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::currentYear,
+      true,
+      i18n("Yearly Budget"),
+      i18n("Default Report")
+    ));
+    list.back().setBudget("Any",false);
+    list.back().setShowingRowTotals(true);
+#ifdef HAVE_KDCHART
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eBudgetActual,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::currentYear,
+      true,
+      i18n("Yearly Budgeted vs Actual Graph"),
+      i18n("Default Report")
+    ));
+    list.back().setChartByDefault(true);
+    list.back().setChartGridLines(false);
+    list.back().setBudget("Any",true);
+    list.back().setDetailLevel(MyMoneyReport::eDetailGroup);
+    list.back().setChartType(MyMoneyReport::eChartLine);
+#endif
+
+    groups.push_back(list);
+  }
+  {
+    ReportGroup list("Forecast", i18n("Forecast"));
+
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eAssetLiability,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::next12Months,
+      false,
+      i18n("Forecast By Month"),
+      i18n("Default Report")
+    ));
+    list.back().setIncludingForecast( true );
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eAssetLiability,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::nextQuarter,
+      false,
+      i18n("Forecast Next Quarter"),
+      i18n("Default Report")
+    ));
+    list.back().setColumnsAreDays( true );
+    list.back().setIncludingForecast( true );
+
+#ifdef HAVE_KDCHART
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eAssetLiability,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::next3Months,
+      false,
+      i18n("Net Worth Forecast Graph"),
+      i18n("Default Report")
+    ));
+    list.back().setColumnsAreDays( true );
+    list.back().setIncludingForecast( true );
+    list.back().setChartByDefault(true);
+    list.back().setChartGridLines(false);
+    list.back().setDetailLevel(MyMoneyReport::eDetailTotal);
+    list.back().setChartType(MyMoneyReport::eChartLine);
+#endif
+    groups.push_back(list);
+  }
+  {
+    ReportGroup list("Information", i18n("General Information"));
+
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eSchedule,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::next12Months,
+      false,
+      i18n("Schedule Information"),
+      i18n("Default Report")
+    ));
+    list.back().setDetailLevel(MyMoneyReport::eDetailAll);
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eSchedule,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::next12Months,
+      false,
+      i18n("Schedule Summary Information"),
+      i18n("Default Report")
+    ));
+    list.back().setDetailLevel(MyMoneyReport::eDetailTop);
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eAccountInfo,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::today,
+      false,
+      i18n("Account Information"),
+      i18n("Default Report")
+    ));
+    list.back().setConvertCurrency(false);
+    list.push_back(MyMoneyReport(
+      MyMoneyReport::eAccountLoanInfo,
+      MyMoneyReport::eMonths,
+      MyMoneyTransactionFilter::today,
+      false,
+      i18n("Loan Information"),
+      i18n("Default Report")
+    ));
+    list.back().setConvertCurrency(false);
     groups.push_back(list);
   }
 }
@@ -894,3 +1411,4 @@ void KReportsView::defaultReports(QValueList<ReportGroup>& groups)
 #undef VIEW_REPORTS
 
 #include "kreportsview.moc"
+// vim:cin:si:ai:et:ts=2:sw=2:
